import BigWorld
import math
from Avatar import PlayerAvatar
from AvatarInputHandler import AvatarInputHandler
from Vehicle import Vehicle
from aih_constants import CTRL_MODE_NAME

import xvm_battle.python.battle as battle
import xvm_main.python.config as config
from xfw.events import registerEvent, overrideMethod
from xfw_actionscript.python import *
from xvm_main.python.logger import *

timeAIM = None
dispersion = None
visible = True
DISPLAY_IN_MODES = [CTRL_MODE_NAME.ARCADE,
                    CTRL_MODE_NAME.ARTY,
                    CTRL_MODE_NAME.DUAL_GUN,
                    CTRL_MODE_NAME.SNIPER,
                    CTRL_MODE_NAME.STRATEGIC,
                    'flamethrower']


@registerEvent(AvatarInputHandler, 'onControlModeChanged')
def AvatarInputHandler_onControlModeChanged(self, eMode, **args):
    global visible
    newVisible = eMode in DISPLAY_IN_MODES
    if newVisible != visible:
        visible = newVisible
        as_event('ON_AIMING')


@registerEvent(Vehicle, '_Vehicle__onAppearanceReady')
def _Vehicle__onAppearanceReady(self, appearance):
    if self.isPlayerVehicle and config.get('sight/enabled', True) and battle.isBattleTypeSupported:
        global timeAIM, visible, dispersion
        visible = True
        timeAIM = None
        dispersion = None


@registerEvent(PlayerAvatar, 'updateVehicleHealth')
def PlayerAvatar_updateVehicleHealth(self, vehicleID, health, deathReasonID, isCrewActive, isRespawn):
    if not (health > 0 and isCrewActive) and config.get('sight/enabled', True) and battle.isBattleTypeSupported:
        global timeAIM, dispersion
        timeAIM = None
        dispersion = None
        as_event('ON_AIMING')


@overrideMethod(PlayerAvatar, 'getOwnVehicleShotDispersionAngle')
def aiming_getOwnVehicleShotDispersionAngle(base, self, turretRotationSpeed, withShot=0):
    global timeAIM, dispersion
    result = base(self, turretRotationSpeed, withShot)
    if config.get('sight/enabled', True) and battle.isBattleTypeSupported and self.isVehicleAlive:
        aimingStartTime, aimingStartFactor, dualAccStartTime, dualAccStartFactor = self._PlayerAvatar__aimingInfo
        multFactor, gunShotDispersionFactorsTurretRotation, chassisShotDispersionFactorsMovement, chassisShotDispersionFactorsRotation, aimingTime = self._PlayerAvatar__dispersionInfo
        dispersion = result[0] * 100
        prevTimeAIM = timeAIM
        aimingTimeAll = math.log(aimingStartFactor / multFactor) * aimingTime
        timeAIM = aimingTimeAll + aimingStartTime - BigWorld.time()
        timeAIM = 0.0 if timeAIM < 0.01 else timeAIM
        try:
            if not (-0.01 < (prevTimeAIM - timeAIM) < 0.01):
                as_event('ON_AIMING')
        except Exception:
            as_event('ON_AIMING')
    return result

@xvm.export('sight.timeAIM', deterministic=False)
def sight_timeAIM():
    return timeAIM if visible else None

@xvm.export('sight.dispersion', deterministic=False)
def sight_timeAIM():
    return dispersion if visible else None
